@@ -15,7 +15,7 @@ import sys
 import weakref
 
 import numpy as np
-import PIL
+import PIL.Image
 
 import matplotlib as mpl
 from matplotlib.backend_bases import (
@@ -24,12 +24,13 @@ from matplotlib.backend_bases import (
     TimerBase, ToolContainerBase, cursors,
     CloseEvent, KeyEvent, LocationEvent, MouseEvent, ResizeEvent)
 
-from matplotlib import _api, cbook, backend_tools
+from matplotlib import _api, cbook, backend_tools, _c_internal_utils
 from matplotlib._pylab_helpers import Gcf
 from matplotlib.path import Path
 from matplotlib.transforms import Affine2D
 
 import wx
+import wx.svg
 
 _log = logging.getLogger(__name__)
 
@@ -39,37 +40,14 @@ _log = logging.getLogger(__name__)
 PIXELS_PER_INCH = 75
 
 
-@_api.caching_module_getattr  # module-level deprecations
-class __getattr__:
-    cursord = _api.deprecated("3.5", obj_type="")(property(lambda self: {
-        cursors.MOVE: wx.CURSOR_HAND,
-        cursors.HAND: wx.CURSOR_HAND,
-        cursors.POINTER: wx.CURSOR_ARROW,
-        cursors.SELECT_REGION: wx.CURSOR_CROSS,
-        cursors.WAIT: wx.CURSOR_WAIT,
-        cursors.RESIZE_HORIZONTAL: wx.CURSOR_SIZEWE,
-        cursors.RESIZE_VERTICAL: wx.CURSOR_SIZENS,
-    }))
-
-
-@_api.deprecated("3.6")
-def error_msg_wx(msg, parent=None):
-    """Signal an error condition with a popup error dialog."""
-    dialog = wx.MessageDialog(parent=parent,
-                              message=msg,
-                              caption='Matplotlib backend_wx error',
-                              style=wx.OK | wx.CENTRE)
-    dialog.ShowModal()
-    dialog.Destroy()
-    return None
-
-
 # lru_cache holds a reference to the App and prevents it from being gc'ed.
 @functools.lru_cache(1)
 def _create_wxapp():
     wxapp = wx.App(False)
     wxapp.SetExitOnFrameDelete(True)
     cbook._setup_new_guiapp()
+    # Set per-process DPI awareness. This is a NoOp except in MSW
+    _c_internal_utils.Win32_SetProcessDpiAwareness_max()
     return wxapp
 
 
@@ -163,10 +141,6 @@ class RendererWx(RendererBase):
         # docstring inherited
         return True
 
-    @_api.deprecated("3.6")
-    def offset_text_height(self):
-        return True
-
     def get_text_width_height_descent(self, s, prop, ismath):
         # docstring inherited
 
@@ -296,7 +270,7 @@ class RendererWx(RendererBase):
         # Font colour is determined by the active wx.Pen
         # TODO: It may be wise to cache font information
         self.fontd[key] = font = wx.Font(  # Cache the font and gc.
-            pointSize=int(size + 0.5),
+            pointSize=round(size),
             family=self.fontnames.get(prop.get_name(), wx.ROMAN),
             style=self.fontangles[prop.get_style()],
             weight=self.fontweights[prop.get_weight()])
@@ -316,7 +290,7 @@ class GraphicsContextWx(GraphicsContextBase):
     seems to be fairly heavy, so these objects are cached based on the
     bitmap object that is passed in.
 
-    The base GraphicsContext stores colors as a RGB tuple on the unit
+    The base GraphicsContext stores colors as an RGB tuple on the unit
     interval, e.g., (0.5, 0.0, 1.0).  wxPython uses an int interval, but
     since wxPython colour management is rather simple, I have not chosen
     to implement a separate colour manager class.
@@ -411,7 +385,7 @@ class GraphicsContextWx(GraphicsContextBase):
         self.unselect()
 
     def get_wxcolour(self, color):
-        """Convert a RGB(A) color to a wx.Colour."""
+        """Convert an RGB(A) color to a wx.Colour."""
         _log.debug("%s - get_wx_color()", type(self))
         return wx.Colour(*[int(255 * x) for x in color])
 
@@ -500,14 +474,16 @@ class _FigureCanvasWxBase(FigureCanvasBase, wx.Panel):
         """
 
         FigureCanvasBase.__init__(self, figure)
-        w, h = map(math.ceil, self.figure.bbox.size)
+        size = wx.Size(*map(math.ceil, self.figure.bbox.size))
+        if wx.Platform != '__WXMSW__':
+            size = parent.FromDIP(size)
         # Set preferred window size hint - helps the sizer, if one is connected
-        wx.Panel.__init__(self, parent, id, size=wx.Size(w, h))
-        # Create the drawing bitmap
-        self.bitmap = wx.Bitmap(w, h)
-        _log.debug("%s - __init__() - bitmap w:%d h:%d", type(self), w, h)
+        wx.Panel.__init__(self, parent, id, size=size)
+        self.bitmap = None
         self._isDrawn = False
         self._rubberband_rect = None
+        self._rubberband_pen_black = wx.Pen('BLACK', 1, wx.PENSTYLE_SHORT_DASH)
+        self._rubberband_pen_white = wx.Pen('WHITE', 1, wx.PENSTYLE_SOLID)
 
         self.Bind(wx.EVT_SIZE, self._on_size)
         self.Bind(wx.EVT_PAINT, self._on_paint)
@@ -539,6 +515,12 @@ class _FigureCanvasWxBase(FigureCanvasBase, wx.Panel):
         self.SetBackgroundStyle(wx.BG_STYLE_PAINT)  # Reduce flicker.
         self.SetBackgroundColour(wx.WHITE)
 
+        if wx.Platform == '__WXMAC__':
+            # Initial scaling. Other platforms handle this automatically
+            dpiScale = self.GetDPIScaleFactor()
+            self.SetInitialSize(self.GetSize()*(1/dpiScale))
+            self._set_device_pixel_ratio(dpiScale)
+
     def Copy_to_Clipboard(self, event=None):
         """Copy bitmap of canvas to system clipboard."""
         bmp_obj = wx.BitmapDataObject()
@@ -548,8 +530,14 @@ class _FigureCanvasWxBase(FigureCanvasBase, wx.Panel):
             open_success = wx.TheClipboard.Open()
             if open_success:
                 wx.TheClipboard.SetData(bmp_obj)
-                wx.TheClipboard.Close()
                 wx.TheClipboard.Flush()
+                wx.TheClipboard.Close()
+
+    def _update_device_pixel_ratio(self, *args, **kwargs):
+        # We need to be careful in cases with mixed resolution displays if
+        # device_pixel_ratio changes.
+        if self._set_device_pixel_ratio(self.GetDPIScaleFactor()):
+            self.draw()
 
     def draw_idle(self):
         # docstring inherited
@@ -595,7 +583,7 @@ class _FigureCanvasWxBase(FigureCanvasBase, wx.Panel):
         for i, (name, exts) in enumerate(sorted_filetypes):
             ext_list = ';'.join(['*.%s' % ext for ext in exts])
             extensions.append(exts[0])
-            wildcard = '%s (%s)|%s' % (name, ext_list, ext_list)
+            wildcard = f'{name} ({ext_list})|{ext_list}'
             if default_filetype in exts:
                 filter_index = i
             wildcards.append(wildcard)
@@ -606,8 +594,6 @@ class _FigureCanvasWxBase(FigureCanvasBase, wx.Panel):
         """
         Update the displayed image on the GUI canvas, using the supplied
         wx.PaintDC device context.
-
-        The 'WXAgg' backend sets origin accordingly.
         """
         _log.debug("%s - gui_repaint()", type(self))
         # The "if self" check avoids a "wrapped C/C++ object has been deleted"
@@ -616,7 +602,7 @@ class _FigureCanvasWxBase(FigureCanvasBase, wx.Panel):
             return
         if not drawDC:  # not called from OnPaint use a ClientDC
             drawDC = wx.ClientDC(self)
-        # For 'WX' backend on Windows, the bitmap can not be in use by another
+        # For 'WX' backend on Windows, the bitmap cannot be in use by another
         # DC (see GraphicsContextWx._cache).
         bmp = (self.bitmap.ConvertToImage().ConvertToBitmap()
                if wx.Platform == '__WXMSW__'
@@ -625,11 +611,11 @@ class _FigureCanvasWxBase(FigureCanvasBase, wx.Panel):
         drawDC.DrawBitmap(bmp, 0, 0)
         if self._rubberband_rect is not None:
             # Some versions of wx+python don't support numpy.float64 here.
-            x0, y0, x1, y1 = map(int, self._rubberband_rect)
-            drawDC.DrawLineList(
-                [(x0, y0, x1, y0), (x1, y0, x1, y1),
-                 (x0, y0, x0, y1), (x0, y1, x1, y1)],
-                wx.Pen('BLACK', 1, wx.PENSTYLE_SHORT_DASH))
+            x0, y0, x1, y1 = map(round, self._rubberband_rect)
+            rect = [(x0, y0, x1, y0), (x1, y0, x1, y1),
+                    (x0, y0, x0, y1), (x0, y1, x1, y1)]
+            drawDC.DrawLineList(rect, self._rubberband_pen_white)
+            drawDC.DrawLineList(rect, self._rubberband_pen_black)
 
     filetypes = {
         **FigureCanvasBase.filetypes,
@@ -643,15 +629,6 @@ class _FigureCanvasWxBase(FigureCanvasBase, wx.Panel):
         'xpm': 'X pixmap',
     }
 
-    def print_figure(self, filename, *args, **kwargs):
-        # docstring inherited
-        super().print_figure(filename, *args, **kwargs)
-        # Restore the current view; this is needed because the artist contains
-        # methods rely on particular attributes of the rendered figure for
-        # determining things like bounding boxes.
-        if self._isDrawn:
-            self.draw()
-
     def _on_paint(self, event):
         """Called when wxPaintEvt is generated."""
         _log.debug("%s - _on_paint()", type(self))
@@ -669,7 +646,7 @@ class _FigureCanvasWxBase(FigureCanvasBase, wx.Panel):
         In this application we attempt to resize to fit the window, so it
         is better to take the performance hit and redraw the whole window.
         """
-
+        self._update_device_pixel_ratio()
         _log.debug("%s - _on_size()", type(self))
         sz = self.GetParent().GetSizer()
         if sz:
@@ -693,9 +670,10 @@ class _FigureCanvasWxBase(FigureCanvasBase, wx.Panel):
             return  # Empty figure
 
         # Create a new, correctly sized bitmap
-        self.bitmap = wx.Bitmap(self._width, self._height)
-
         dpival = self.figure.dpi
+        if not wx.Platform == '__WXMSW__':
+            scale = self.GetDPIScaleFactor()
+            dpival /= scale
         winch = self._width / dpival
         hinch = self._height / dpival
         self.figure.set_size_inches(winch, hinch, forward=False)
@@ -707,8 +685,22 @@ class _FigureCanvasWxBase(FigureCanvasBase, wx.Panel):
         ResizeEvent("resize_event", self)._process()
         self.draw_idle()
 
-    def _get_key(self, event):
+    @staticmethod
+    def _mpl_modifiers(event=None, *, exclude=None):
+        mod_table = [
+            ("ctrl", wx.MOD_CONTROL, wx.WXK_CONTROL),
+            ("alt", wx.MOD_ALT, wx.WXK_ALT),
+            ("shift", wx.MOD_SHIFT, wx.WXK_SHIFT),
+        ]
+        if event is not None:
+            modifiers = event.GetModifiers()
+            return [name for name, mod, key in mod_table
+                    if modifiers & mod and exclude != key]
+        else:
+            return [name for name, mod, key in mod_table
+                    if wx.GetKeyState(key)]
 
+    def _get_key(self, event):
         keyval = event.KeyCode
         if keyval in self.keyvald:
             key = self.keyvald[keyval]
@@ -719,18 +711,11 @@ class _FigureCanvasWxBase(FigureCanvasBase, wx.Panel):
             if not event.ShiftDown():
                 key = key.lower()
         else:
-            key = None
-
-        for meth, prefix, key_name in [
-                (event.ControlDown, 'ctrl', 'control'),
-                (event.AltDown, 'alt', 'alt'),
-                (event.ShiftDown, 'shift', 'shift'),
-        ]:
-            if meth() and key_name != key:
-                if not (key_name == 'shift' and key.isupper()):
-                    key = '{0}+{1}'.format(prefix, key)
-
-        return key
+            return None
+        mods = self._mpl_modifiers(event, exclude=keyval)
+        if "shift" in mods and key.isupper():
+            mods.remove("shift")
+        return "+".join([*mods, key])
 
     def _mpl_coords(self, pos=None):
         """
@@ -743,7 +728,11 @@ class _FigureCanvasWxBase(FigureCanvasBase, wx.Panel):
         else:
             x, y = pos.X, pos.Y
         # flip y so y=0 is bottom of canvas
-        return x, self.figure.bbox.height - y
+        if not wx.Platform == '__WXMSW__':
+            scale = self.GetDPIScaleFactor()
+            return x*scale, self.figure.bbox.height - y*scale
+        else:
+            return x, self.figure.bbox.height - y
 
     def _on_key_down(self, event):
         """Capture key press."""
@@ -800,15 +789,17 @@ class _FigureCanvasWxBase(FigureCanvasBase, wx.Panel):
         }
         button = event.GetButton()
         button = button_map.get(button, button)
+        modifiers = self._mpl_modifiers(event)
         if event.ButtonDown():
-            MouseEvent("button_press_event", self,
-                       x, y, button, guiEvent=event)._process()
+            MouseEvent("button_press_event", self, x, y, button,
+                       modifiers=modifiers, guiEvent=event)._process()
         elif event.ButtonDClick():
-            MouseEvent("button_press_event", self,
-                       x, y, button, dblclick=True, guiEvent=event)._process()
+            MouseEvent("button_press_event", self, x, y, button,
+                       dblclick=True, modifiers=modifiers,
+                       guiEvent=event)._process()
         elif event.ButtonUp():
-            MouseEvent("button_release_event", self,
-                       x, y, button, guiEvent=event)._process()
+            MouseEvent("button_release_event", self, x, y, button,
+                       modifiers=modifiers, guiEvent=event)._process()
 
     def _on_mouse_wheel(self, event):
         """Translate mouse wheel events into matplotlib events"""
@@ -826,14 +817,16 @@ class _FigureCanvasWxBase(FigureCanvasBase, wx.Panel):
                 return  # Return without processing event
             else:
                 self._skipwheelevent = True
-        MouseEvent("scroll_event", self,
-                   x, y, step=step, guiEvent=event)._process()
+        MouseEvent("scroll_event", self, x, y, step=step,
+                   modifiers=self._mpl_modifiers(event),
+                   guiEvent=event)._process()
 
     def _on_motion(self, event):
         """Start measuring on an axis."""
         event.Skip()
         MouseEvent("motion_notify_event", self,
                    *self._mpl_coords(event),
+                   modifiers=self._mpl_modifiers(event),
                    guiEvent=event)._process()
 
     def _on_enter(self, event):
@@ -841,6 +834,7 @@ class _FigureCanvasWxBase(FigureCanvasBase, wx.Panel):
         event.Skip()
         LocationEvent("figure_enter_event", self,
                       *self._mpl_coords(event),
+                      modifiers=self._mpl_modifiers(),
                       guiEvent=event)._process()
 
     def _on_leave(self, event):
@@ -848,6 +842,7 @@ class _FigureCanvasWxBase(FigureCanvasBase, wx.Panel):
         event.Skip()
         LocationEvent("figure_leave_event", self,
                       *self._mpl_coords(event),
+                      modifiers=self._mpl_modifiers(),
                       guiEvent=event)._process()
 
 
@@ -900,7 +895,7 @@ class FigureCanvasWx(_FigureCanvasWxBase):
 
 
 class FigureFrameWx(wx.Frame):
-    def __init__(self, num, fig, *, canvas_class=None):
+    def __init__(self, num, fig, *, canvas_class):
         # On non-Windows platform, explicitly set the position - fix
         # positioning bug on some Linux platforms
         if wx.Platform == '__WXMSW__':
@@ -912,16 +907,7 @@ class FigureFrameWx(wx.Frame):
         _log.debug("%s - __init__()", type(self))
         _set_frame_icon(self)
 
-        # The parameter will become required after the deprecation elapses.
-        if canvas_class is not None:
-            self.canvas = canvas_class(self, -1, fig)
-        else:
-            _api.warn_deprecated(
-                "3.6", message="The canvas_class parameter will become "
-                "required after the deprecation period starting in Matplotlib "
-                "%(since)s elapses.")
-            self.canvas = self.get_canvas(fig)
-
+        self.canvas = canvas_class(self, -1, fig)
         # Auto-attaches itself to self.canvas.manager
         manager = FigureManagerWx(self.canvas, num, self)
 
@@ -932,36 +918,14 @@ class FigureFrameWx(wx.Frame):
         # On Windows, canvas sizing must occur after toolbar addition;
         # otherwise the toolbar further resizes the canvas.
         w, h = map(math.ceil, fig.bbox.size)
-        self.canvas.SetInitialSize(wx.Size(w, h))
-        self.canvas.SetMinSize((2, 2))
+        self.canvas.SetInitialSize(self.FromDIP(wx.Size(w, h)))
+        self.canvas.SetMinSize(self.FromDIP(wx.Size(2, 2)))
         self.canvas.SetFocus()
 
         self.Fit()
 
         self.Bind(wx.EVT_CLOSE, self._on_close)
 
-    sizer = _api.deprecated("3.6", alternative="frame.GetSizer()")(
-        property(lambda self: self.GetSizer()))
-    figmgr = _api.deprecated("3.6", alternative="frame.canvas.manager")(
-        property(lambda self: self.canvas.manager))
-    num = _api.deprecated("3.6", alternative="frame.canvas.manager.num")(
-        property(lambda self: self.canvas.manager.num))
-    toolbar = _api.deprecated("3.6", alternative="frame.GetToolBar()")(
-        property(lambda self: self.GetToolBar()))
-    toolmanager = _api.deprecated(
-        "3.6", alternative="frame.canvas.manager.toolmanager")(
-            property(lambda self: self.canvas.manager.toolmanager))
-
-    @_api.deprecated(
-        "3.6", alternative="the canvas_class constructor parameter")
-    def get_canvas(self, fig):
-        return FigureCanvasWx(self, -1, fig)
-
-    @_api.deprecated("3.6", alternative="frame.canvas.manager")
-    def get_figure_manager(self):
-        _log.debug("%s - get_figure_manager()", type(self))
-        return self.canvas.manager
-
     def _on_close(self, event):
         _log.debug("%s - on_close()", type(self))
         CloseEvent("close_event", self.canvas)._process()
@@ -1010,6 +974,13 @@ class FigureManagerWx(FigureManagerBase):
             figure.canvas.draw_idle()
         return manager
 
+    @classmethod
+    def start_main_loop(cls):
+        if not wx.App.IsMainLoopRunning():
+            wxapp = wx.GetApp()
+            if wxapp is not None:
+                wxapp.MainLoop()
+
     def show(self):
         # docstring inherited
         self.frame.Show()
@@ -1066,9 +1037,9 @@ def _set_frame_icon(frame):
 class NavigationToolbar2Wx(NavigationToolbar2, wx.ToolBar):
     def __init__(self, canvas, coordinates=True, *, style=wx.TB_BOTTOM):
         wx.ToolBar.__init__(self, canvas.GetParent(), -1, style=style)
+        if wx.Platform == '__WXMAC__':
+            self.SetToolBitmapSize(self.GetToolBitmapSize()*self.GetDPIScaleFactor())
 
-        if 'wxMac' in wx.PlatformInfo:
-            self.SetToolBitmapSize((24, 24))
         self.wx_ids = {}
         for text, tooltip_text, image_file, callback in self.toolitems:
             if text is None:
@@ -1077,7 +1048,7 @@ class NavigationToolbar2Wx(NavigationToolbar2, wx.ToolBar):
             self.wx_ids[text] = (
                 self.AddTool(
                     -1,
-                    bitmap=self._icon(f"{image_file}.png"),
+                    bitmap=self._icon(f"{image_file}.svg"),
                     bmpDisabled=wx.NullBitmap,
                     label=text, shortHelp=tooltip_text,
                     kind=(wx.ITEM_CHECK if text in ["Pan", "Zoom"]
@@ -1103,7 +1074,6 @@ class NavigationToolbar2Wx(NavigationToolbar2, wx.ToolBar):
         *name*, including the extension and relative to Matplotlib's "images"
         data directory.
         """
-        image = np.array(PIL.Image.open(cbook._get_data_path("images", name)))
         try:
             dark = wx.SystemSettings.GetAppearance().IsDark()
         except AttributeError:  # wxpython < 4.1
@@ -1114,12 +1084,24 @@ class NavigationToolbar2Wx(NavigationToolbar2, wx.ToolBar):
             bg_lum = (.299 * bg.red + .587 * bg.green + .114 * bg.blue) / 255
             fg_lum = (.299 * fg.red + .587 * fg.green + .114 * fg.blue) / 255
             dark = fg_lum - bg_lum > .2
-        if dark:
-            fg = wx.SystemSettings.GetColour(wx.SYS_COLOUR_WINDOWTEXT)
-            black_mask = (image[..., :3] == 0).all(axis=-1)
-            image[black_mask, :3] = (fg.Red(), fg.Green(), fg.Blue())
-        return wx.Bitmap.FromBufferRGBA(
-            image.shape[1], image.shape[0], image.tobytes())
+
+        path = cbook._get_data_path('images', name)
+        if path.suffix == '.svg':
+            svg = path.read_bytes()
+            if dark:
+                svg = svg.replace(b'fill:black;', b'fill:white;')
+            toolbarIconSize = wx.ArtProvider().GetDIPSizeHint(wx.ART_TOOLBAR)
+            return wx.BitmapBundle.FromSVG(svg, toolbarIconSize)
+        else:
+            pilimg = PIL.Image.open(path)
+            # ensure RGBA as wx BitMap expects RGBA format
+            image = np.array(pilimg.convert("RGBA"))
+            if dark:
+                fg = wx.SystemSettings.GetColour(wx.SYS_COLOUR_WINDOWTEXT)
+                black_mask = (image[..., :3] == 0).all(axis=-1)
+                image[black_mask, :3] = (fg.Red(), fg.Green(), fg.Blue())
+            return wx.Bitmap.FromBufferRGBA(
+                image.shape[1], image.shape[0], image.tobytes())
 
     def _update_buttons_checked(self):
         if "Pan" in self.wx_ids:
@@ -1160,7 +1142,7 @@ class NavigationToolbar2Wx(NavigationToolbar2, wx.ToolBar):
             if mpl.rcParams["savefig.directory"]:
                 mpl.rcParams["savefig.directory"] = str(path.parent)
             try:
-                self.canvas.figure.savefig(str(path), format=fmt)
+                self.canvas.figure.savefig(path, format=fmt)
             except Exception as e:
                 dialog = wx.MessageDialog(
                     parent=self.canvas.GetParent(), message=str(e),
@@ -1170,7 +1152,9 @@ class NavigationToolbar2Wx(NavigationToolbar2, wx.ToolBar):
 
     def draw_rubberband(self, event, x0, y0, x1, y1):
         height = self.canvas.figure.bbox.height
-        self.canvas._rubberband_rect = (x0, height - y0, x1, height - y1)
+        sf = 1 if wx.Platform == '__WXMSW__' else self.canvas.GetDPIScaleFactor()
+        self.canvas._rubberband_rect = (x0/sf, (height - y0)/sf,
+                                        x1/sf, (height - y1)/sf)
         self.canvas.Refresh()
 
     def remove_rubberband(self):
@@ -1183,7 +1167,7 @@ class NavigationToolbar2Wx(NavigationToolbar2, wx.ToolBar):
 
     def set_history_buttons(self):
         can_backward = self._nav_stack._pos > 0
-        can_forward = self._nav_stack._pos < len(self._nav_stack._elements) - 1
+        can_forward = self._nav_stack._pos < len(self._nav_stack) - 1
         if 'Back' in self.wx_ids:
             self.EnableTool(self.wx_ids['Back'], can_backward)
         if 'Forward' in self.wx_ids:
@@ -1193,6 +1177,8 @@ class NavigationToolbar2Wx(NavigationToolbar2, wx.ToolBar):
 # tools for matplotlib.backend_managers.ToolManager:
 
 class ToolbarWx(ToolContainerBase, wx.ToolBar):
+    _icon_extension = '.svg'
+
     def __init__(self, toolmanager, parent=None, style=wx.TB_BOTTOM):
         if parent is None:
             parent = toolmanager.canvas.GetParent()
@@ -1271,9 +1257,8 @@ class ToolbarWx(ToolContainerBase, wx.ToolBar):
         self.Refresh()
 
     def remove_toolitem(self, name):
-        for tool, handler in self._toolitems[name]:
+        for tool, handler in self._toolitems.pop(name, []):
             self.DeleteTool(tool.Id)
-        del self._toolitems[name]
 
     def set_message(self, s):
         self._label_text.SetLabel(s)
@@ -1292,13 +1277,6 @@ class SaveFigureWx(backend_tools.SaveFigureBase):
             self._make_classic_style_pseudo_toolbar())
 
 
-@_api.deprecated("3.5", alternative="ToolSetCursor")
-class SetCursorWx(backend_tools.SetCursorBase):
-    def set_cursor(self, cursor):
-        NavigationToolbar2Wx.set_cursor(
-            self._make_classic_style_pseudo_toolbar(), cursor)
-
-
 @backend_tools._register_tool_class(_FigureCanvasWxBase)
 class RubberbandWx(backend_tools.RubberbandBase):
     def draw_rubberband(self, x0, y0, x1, y1):
@@ -1383,10 +1361,4 @@ FigureManagerWx._toolmanager_toolbar_class = ToolbarWx
 class _BackendWx(_Backend):
     FigureCanvas = FigureCanvasWx
     FigureManager = FigureManagerWx
-
-    @staticmethod
-    def mainloop():
-        if not wx.App.IsMainLoopRunning():
-            wxapp = wx.GetApp()
-            if wxapp is not None:
-                wxapp.MainLoop()
+    mainloop = FigureManagerWx.start_main_loop
